@@ -177,7 +177,7 @@ class nsComboButtonListener: public nsIDOMMouseListener
     PRBool isDroppedDown;
     mComboBox->IsDroppedDown(&isDroppedDown);
     mComboBox->ShowDropDown(!isDroppedDown);
-    return PR_FALSE; 
+    return NS_OK; 
   }
 
   nsComboButtonListener(nsComboboxControlFrame* aCombobox) 
@@ -345,7 +345,7 @@ nsComboboxControlFrame::nsComboboxControlFrame()
   mInRedisplayText = PR_FALSE;
   mRedisplayTextEventPosted = PR_FALSE;
 
-  mRecentSelectedIndex = -1;
+  mRecentSelectedIndex = NS_SKIP_NOTIFY_INDEX;
 
   //Shrink the area around it's contents
   //SetFlags(NS_BLOCK_SHRINK_WRAP);
@@ -508,17 +508,26 @@ nsComboboxControlFrame::GetHorizontalInsidePadding(nsPresContext* aPresContext,
 void 
 nsComboboxControlFrame::SetFocus(PRBool aOn, PRBool aRepaint)
 {
+  nsWeakFrame weakFrame(this);
   if (aOn) {
     nsListControlFrame::ComboboxFocusSet();
     mFocused = this;
   } else {
     mFocused = nsnull;
     if (mDroppedDown) {
-      mListControlFrame->ComboboxFinish(mDisplayedIndex);
+      mListControlFrame->ComboboxFinish(mDisplayedIndex); // might destroy us
+      if (!weakFrame.IsAlive()) {
+        return;
+      }
     }
+    // May delete |this|.
     mListControlFrame->FireOnChange();
   }
 
+  if (!weakFrame.IsAlive()) {
+    return;
+  }
+
   // This is needed on a temporary basis. It causes the focus
   // rect to be drawn. This is much faster than ReResolvingStyle
   // Bug 32920
@@ -571,49 +580,50 @@ nsComboboxControlFrame::ShowPopup(PRBool aShowPopup)
                      NS_XUL_POPUP_SHOWING : NS_XUL_POPUP_HIDING, nsnull,
                      nsMouseEvent::eReal);
 
-  nsIPresShell *shell = mPresContext->GetPresShell();
+  nsCOMPtr<nsIPresShell> shell = mPresContext->GetPresShell();
   if (shell) 
     shell->HandleDOMEventWithTarget(mContent, &event, &status);
 }
 
-// Show the dropdown list
-
-void 
+PRBool
 nsComboboxControlFrame::ShowList(nsPresContext* aPresContext, PRBool aShowList)
 {
-  nsIWidget* widget = nsnull;
+  nsCOMPtr<nsIPresShell> shell = aPresContext->GetPresShell();
 
-  // Get parent view
-  nsIFrame * listFrame;
-  if (NS_OK == mListControlFrame->QueryInterface(NS_GET_IID(nsIFrame), (void **)&listFrame)) {
-    nsIView* view = listFrame->GetView();
-    NS_ASSERTION(view, "nsComboboxControlFrame view is null");
-    if (view) {
-    	widget = view->GetWidget();
-    }
+  nsWeakFrame weakFrame(this);
+  ShowPopup(aShowList);  // might destroy us
+  if (!weakFrame.IsAlive()) {
+    return PR_FALSE;
   }
-
-  if (PR_TRUE == aShowList) {
-    ShowPopup(PR_TRUE);
-    mDroppedDown = PR_TRUE;
-
-     // The listcontrol frame will call back to the nsComboboxControlFrame's ListWasSelected
-     // which will stop the capture.
+  
+  mDroppedDown = aShowList;
+  if (mDroppedDown) {
+    // The listcontrol frame will call back to the nsComboboxControlFrame's
+    // ListWasSelected which will stop the capture.
     mListControlFrame->AboutToDropDown();
     mListControlFrame->CaptureMouseEvents(aPresContext, PR_TRUE);
-
-  } else {
-    ShowPopup(PR_FALSE);
-    mDroppedDown = PR_FALSE;
   }
 
   // Don't flush anything but reflows lest it destroy us
-  aPresContext->PresShell()->
-    GetDocument()->FlushPendingNotifications(Flush_OnlyReflow);
+  shell->GetDocument()->FlushPendingNotifications(Flush_OnlyReflow);
+  if (!weakFrame.IsAlive()) {
+    NS_ERROR("Flush_OnlyReflow destroyed the frame");
+    return PR_FALSE;
+  }
 
-  if (widget)
-    widget->CaptureRollupEvents((nsIRollupListener *)this, mDroppedDown, aShowList);
+  nsIFrame* listFrame = nsnull;
+  CallQueryInterface(mListControlFrame, &listFrame);
+  if (listFrame) {
+    nsIView* view = listFrame->GetView();
+    NS_ASSERTION(view, "nsComboboxControlFrame view is null");
+    if (view) {
+      nsIWidget* widget = view->GetWidget();
+      if (widget)
+        widget->CaptureRollupEvents(this, mDroppedDown, mDroppedDown);
+    }
+  }
 
+  return weakFrame.IsAlive();
 }
 
 nsresult
@@ -1569,28 +1579,30 @@ nsComboboxControlFrame::Reflow(nsPresContext*          aPresContext,
                      mDisplayFrame, mButtonFrame, mItemDisplayWidth, scrollbarWidth, 
                      aReflowState.mComputedBorderPadding, size.height);
 
-  // The dropdown was reflowed UNCONSTRAINED before, now we need to check to see
-  // if it needs to be resized. 
-  //
-  // Optimization - The style (font, etc.) maybe different for the display item 
-  // than for any particular item in the dropdown. So, if the new size of combobox
-  // is smaller than the dropdown, that is OK, The dropdown MUST always be either the same
-  // size as the combo or larger if necessary
-  if (aDesiredSize.width > dropdownDesiredSize.width) {
-    if (eReflowReason_Initial == firstPassState.reason) {
-      firstPassState.reason = eReflowReason_Resize;
-    }
-    REFLOW_DEBUG_MSG3("*** Reflowing ListBox to width: %d it was %d\n", PX(aDesiredSize.width), PX(dropdownDesiredSize.width));
-
-    // Tell it we are doing the second pass, which means we will skip
-    // doing the unconstained reflow, we already know that size
-    nsListControlFrame * lcf = NS_STATIC_CAST(nsListControlFrame*, mDropdownFrame);
-    lcf->SetPassId(2);
-    // Reflow the dropdown list to match the width of the display + button
-    ReflowComboChildFrame(mDropdownFrame, aPresContext, dropdownDesiredSize, firstPassState, aStatus, 
-                          aDesiredSize.width, NS_UNCONSTRAINEDSIZE);
-    lcf->SetPassId(0);
+  // The dropdown was reflowed UNCONSTRAINED before, now we need to reflow it
+  // so that all children match the desired width.
+  // The dropdown MUST always be either the same size as the combo or larger
+  // if necessary. Note that individual children can be narrower in case they
+  // are constrained by 'width', 'max-width' etc.
+  if (eReflowReason_Initial == firstPassState.reason) {
+    firstPassState.reason = eReflowReason_Resize;
   }
+  REFLOW_DEBUG_MSG3("*** Reflowing ListBox to width: %d it was %d\n", PX(aDesiredSize.width), PX(dropdownDesiredSize.width));
+
+  // Tell it we are doing the second pass, which means we will skip
+  // doing the unconstained reflow, we already know that size
+  nsListControlFrame * lcf = NS_STATIC_CAST(nsListControlFrame*, mDropdownFrame);
+  lcf->SetPassId(2);
+  // Reflow the dropdown list to be
+  // MAX(width of the display + button, width of the widest option). bug 305705.
+  const nscoord availableWidth =
+    PR_MAX(aDesiredSize.width, dropdownDesiredSize.width -
+                               aReflowState.mComputedBorderPadding.left -
+                               aReflowState.mComputedBorderPadding.right);
+  ReflowComboChildFrame(mDropdownFrame, aPresContext, dropdownDesiredSize,
+                        firstPassState, aStatus,
+                        availableWidth, NS_UNCONSTRAINEDSIZE);
+  lcf->SetPassId(0);
 
   // Set the max element size to be the same as the desired element size.
   if (aDesiredSize.mComputeMEW) {
@@ -1705,10 +1717,10 @@ nsComboboxControlFrame::ShowDropDown(PRBool aDoDropDown)
     if (mListControlFrame) {
       mListControlFrame->SyncViewWithFrame();
     }
-    ToggleList(mPresContext);
+    ShowList(mPresContext, aDoDropDown); // might destroy us
     return NS_OK;
   } else if (mDroppedDown && !aDoDropDown) {
-    ToggleList(mPresContext);
+    ShowList(mPresContext, aDoDropDown); // might destroy us
     return NS_OK;
   }
 
@@ -1739,16 +1751,6 @@ nsComboboxControlFrame::GetDropDown(nsIFrame** aDropDownFrame)
   return NS_OK;
 }
 
-// Toggle dropdown list.
-
-NS_IMETHODIMP 
-nsComboboxControlFrame::ToggleList(nsPresContext* aPresContext)
-{
-  ShowList(aPresContext, (PR_FALSE == mDroppedDown));
-
-  return NS_OK;
-}
-
 NS_IMETHODIMP
 nsComboboxControlFrame::AbsolutelyPositionDropDown()
 {
@@ -2102,8 +2104,10 @@ nsComboboxControlFrame::CreateAnonymousContent(nsPresContext* aPresContext,
 
   // Add a child text content node for the label
 
+  nsNodeInfoManager *nimgr = mContent->GetNodeInfo()->NodeInfoManager();
+
   nsCOMPtr<nsITextContent> labelContent;
-  NS_NewTextNode(getter_AddRefs(labelContent));
+  NS_NewTextNode(getter_AddRefs(labelContent), nimgr);
 
   if (labelContent) {
     // set the value of the text node
@@ -2114,12 +2118,9 @@ nsComboboxControlFrame::CreateAnonymousContent(nsPresContext* aPresContext,
     }
     ActuallyDisplayText(PR_FALSE);
 
-    nsCOMPtr<nsIDocument> doc = mContent->GetDocument();
-
     nsCOMPtr<nsINodeInfo> nodeInfo;
-    doc->NodeInfoManager()->GetNodeInfo(nsHTMLAtoms::input, nsnull,
-                                        kNameSpaceID_None,
-                                        getter_AddRefs(nodeInfo));
+    nimgr->GetNodeInfo(nsHTMLAtoms::input, nsnull, kNameSpaceID_None,
+                       getter_AddRefs(nodeInfo));
 
     aChildList.AppendElement(mDisplayContent);
 
@@ -2246,7 +2247,7 @@ nsComboboxControlFrame::Destroy(nsPresContext* aPresContext)
       if (view) {
     	  nsIWidget* widget = view->GetWidget();
         if (widget)
-          widget->CaptureRollupEvents((nsIRollupListener *)this, PR_FALSE, PR_TRUE);
+          widget->CaptureRollupEvents(this, PR_FALSE, PR_TRUE);
       }
     }
   }
@@ -2335,8 +2336,13 @@ NS_IMETHODIMP
 nsComboboxControlFrame::Rollup()
 {
   if (mDroppedDown) {
-    mListControlFrame->AboutToRollup();
-    ShowDropDown(PR_FALSE);
+    nsWeakFrame weakFrame(this);
+    mListControlFrame->AboutToRollup(); // might destroy us
+    if (!weakFrame.IsAlive())
+      return NS_OK;
+    ShowDropDown(PR_FALSE); // might destroy us
+    if (!weakFrame.IsAlive())
+      return NS_OK;
     mListControlFrame->CaptureMouseEvents(mPresContext, PR_FALSE);
   }
   return NS_OK;
@@ -2345,9 +2351,8 @@ nsComboboxControlFrame::Rollup()
 NS_IMETHODIMP
 nsComboboxControlFrame::RollupFromList(nsPresContext* aPresContext)
 {
-  ShowList(aPresContext, PR_FALSE);
-  mListControlFrame->CaptureMouseEvents(aPresContext, PR_FALSE);
-
+  if (ShowList(aPresContext, PR_FALSE))
+    mListControlFrame->CaptureMouseEvents(aPresContext, PR_FALSE);
   return NS_OK;
 }
 
@@ -2355,7 +2360,7 @@ NS_IMETHODIMP_(PRInt32)
 nsComboboxControlFrame::UpdateRecentIndex(PRInt32 aIndex)
 {
   PRInt32 index = mRecentSelectedIndex;
-  if (mRecentSelectedIndex == -1 || aIndex == -1)
+  if (mRecentSelectedIndex == NS_SKIP_NOTIFY_INDEX || aIndex == NS_SKIP_NOTIFY_INDEX)
     mRecentSelectedIndex = aIndex;
   return index;
 }
